@@ -1,6 +1,6 @@
 import itertools
 import warnings
-from collections import Counter, OrderedDict
+from collections import Counter
 from textwrap import dedent
 
 import pandas as pd
@@ -13,7 +13,7 @@ from .merge import merge
 
 
 def _infer_concat_order_from_positions(datasets):
-    combined_ids = OrderedDict(_infer_tile_ids_from_nested_list(datasets, ()))
+    combined_ids = dict(_infer_tile_ids_from_nested_list(datasets, ()))
     return combined_ids
 
 
@@ -110,7 +110,7 @@ def _infer_concat_order_from_coords(datasets):
             "order the datasets for concatenation"
         )
 
-    combined_ids = OrderedDict(zip(tile_ids, datasets))
+    combined_ids = dict(zip(tile_ids, datasets))
 
     return combined_ids, concat_dims
 
@@ -207,14 +207,14 @@ def _combine_all_along_first_dim(
 
     # Group into lines of datasets which must be combined along dim
     # need to sort by _new_tile_id first for groupby to work
-    # TODO remove all these sorted OrderedDicts once python >= 3.6 only
-    combined_ids = OrderedDict(sorted(combined_ids.items(), key=_new_tile_id))
+    # TODO: is the sorted need?
+    combined_ids = dict(sorted(combined_ids.items(), key=_new_tile_id))
     grouped = itertools.groupby(combined_ids.items(), key=_new_tile_id)
 
     # Combine all of these datasets along dim
     new_combined_ids = {}
     for new_id, group in grouped:
-        combined_ids = OrderedDict(sorted(group))
+        combined_ids = dict(sorted(group))
         datasets = combined_ids.values()
         new_combined_ids[new_id] = _combine_1d(
             datasets, dim, compat, data_vars, coords, fill_value, join
@@ -291,7 +291,7 @@ def _nested_combine(
         combined_ids = _infer_concat_order_from_positions(datasets)
     else:
         # Already sorted so just use the ids already passed
-        combined_ids = OrderedDict(zip(ids, datasets))
+        combined_ids = dict(zip(ids, datasets))
 
     # Check that the inferred shape is combinable
     _check_shape_tile_ids(combined_ids)
@@ -393,7 +393,7 @@ def combine_nested(
     --------
 
     A common task is collecting data from a parallelized simulation in which
-    each processor wrote out to a separate file. A domain which was decomposed
+    each process wrote out to a separate file. A domain which was decomposed
     into 4 parts, 2 each along both the x and y axes, requires organising the
     datasets into a doubly-nested list, e.g:
 
@@ -505,8 +505,7 @@ def combine_by_coords(
     ----------
     datasets : sequence of xarray.Dataset
         Dataset objects to combine.
-    compat : {'identical', 'equals', 'broadcast_equals',
-              'no_conflicts', 'override'}, optional
+    compat : {'identical', 'equals', 'broadcast_equals', 'no_conflicts', 'override'}, optional
         String indicating how to compare variables of the same name for
         potential conflicts:
 
@@ -520,9 +519,22 @@ def combine_by_coords(
           of all non-null values.
         - 'override': skip comparing and pick variable from first dataset
     data_vars : {'minimal', 'different', 'all' or list of str}, optional
-        Details are in the documentation of concat
+        These data variables will be concatenated together:
+
+        * 'minimal': Only data variables in which the dimension already
+          appears are included.
+        * 'different': Data variables which are not equal (ignoring
+          attributes) across all datasets are also concatenated (as well as
+          all for which dimension already appears). Beware: this option may
+          load the data payload of data variables into memory if they are not
+          already loaded.
+        * 'all': All data variables will be concatenated.
+        * list of str: The listed data variables will be concatenated, in
+          addition to the 'minimal' data variables.
+
+        If objects are DataArrays, `data_vars` must be 'all'.
     coords : {'minimal', 'different', 'all' or list of str}, optional
-        Details are in the documentation of concat
+        As per the 'data_vars' kwarg, but for coordinate variables.
     fill_value : scalar, optional
         Value to use for newly missing values
     join : {'outer', 'inner', 'left', 'right', 'exact'}, optional
@@ -556,29 +568,91 @@ def combine_by_coords(
     they are concatenated based on the values in their dimension coordinates,
     not on their position in the list passed to `combine_by_coords`.
 
+    >>> import numpy as np
+    >>> import xarray as xr
+
+    >>> x1 = xr.Dataset(
+    ...     {
+    ...         "temperature": (("y", "x"), 20 * np.random.rand(6).reshape(2, 3)),
+    ...         "precipitation": (("y", "x"), np.random.rand(6).reshape(2, 3)),
+    ...     },
+    ...     coords={"y": [0, 1], "x": [10, 20, 30]},
+    ... )
+    >>> x2 = xr.Dataset(
+    ...     {
+    ...         "temperature": (("y", "x"), 20 * np.random.rand(6).reshape(2, 3)),
+    ...         "precipitation": (("y", "x"), np.random.rand(6).reshape(2, 3)),
+    ...     },
+    ...     coords={"y": [2, 3], "x": [10, 20, 30]},
+    ... )
+    >>> x3 = xr.Dataset(
+    ...     {
+    ...         "temperature": (("y", "x"), 20 * np.random.rand(6).reshape(2, 3)),
+    ...         "precipitation": (("y", "x"), np.random.rand(6).reshape(2, 3)),
+    ...     },
+    ...     coords={"y": [2, 3], "x": [40, 50, 60]},
+    ... )
+
     >>> x1
     <xarray.Dataset>
-    Dimensions:         (x: 3)
-    Coords:
-      * position        (x) int64   0 1 2
+    Dimensions:        (x: 3, y: 2)
+    Coordinates:
+    * y              (y) int64 0 1
+    * x              (x) int64 10 20 30
     Data variables:
-        temperature     (x) float64 11.04 23.57 20.77 ...
+        temperature    (y, x) float64 1.654 10.63 7.015 2.543 13.93 9.436
+        precipitation  (y, x) float64 0.2136 0.9974 0.7603 0.4679 0.3115 0.945
 
     >>> x2
     <xarray.Dataset>
-    Dimensions:         (x: 3)
-    Coords:
-      * position        (x) int64   3 4 5
+    Dimensions:        (x: 3, y: 2)
+    Coordinates:
+    * y              (y) int64 2 3
+    * x              (x) int64 10 20 30
     Data variables:
-        temperature     (x) float64 6.97 8.13 7.42 ...
+        temperature    (y, x) float64 9.341 0.1251 6.269 7.709 8.82 2.316
+        precipitation  (y, x) float64 0.1728 0.1178 0.03018 0.6509 0.06938 0.3792
 
-    >>> combined = xr.combine_by_coords([x2, x1])
+    >>> x3
     <xarray.Dataset>
-    Dimensions:         (x: 6)
-    Coords:
-      * position        (x) int64   0 1 2 3 4 5
+    Dimensions:        (x: 3, y: 2)
+    Coordinates:
+    * y              (y) int64 2 3
+    * x              (x) int64 40 50 60
     Data variables:
-        temperature     (x) float64 11.04 23.57 20.77 ...
+        temperature    (y, x) float64 2.789 2.446 6.551 12.46 2.22 15.96
+        precipitation  (y, x) float64 0.4804 0.1902 0.2457 0.6125 0.4654 0.5953
+
+    >>> xr.combine_by_coords([x2, x1])
+    <xarray.Dataset>
+    Dimensions:        (x: 3, y: 4)
+    Coordinates:
+    * x              (x) int64 10 20 30
+    * y              (y) int64 0 1 2 3
+    Data variables:
+        temperature    (y, x) float64 1.654 10.63 7.015 2.543 ... 7.709 8.82 2.316
+        precipitation  (y, x) float64 0.2136 0.9974 0.7603 ... 0.6509 0.06938 0.3792
+
+    >>> xr.combine_by_coords([x3, x1])
+    <xarray.Dataset>
+    Dimensions:        (x: 6, y: 4)
+    Coordinates:
+    * x              (x) int64 10 20 30 40 50 60
+    * y              (y) int64 0 1 2 3
+    Data variables:
+        temperature    (y, x) float64 1.654 10.63 7.015 nan ... nan 12.46 2.22 15.96
+        precipitation  (y, x) float64 0.2136 0.9974 0.7603 ... 0.6125 0.4654 0.5953
+
+    >>> xr.combine_by_coords([x3, x1], join='override')
+    <xarray.Dataset>
+    Dimensions:        (x: 3, y: 4)
+    Coordinates:
+    * x              (x) int64 10 20 30
+    * y              (y) int64 0 1 2 3
+    Data variables:
+    temperature    (y, x) float64 1.654 10.63 7.015 2.543 ... 12.46 2.22 15.96
+    precipitation  (y, x) float64 0.2136 0.9974 0.7603 ... 0.6125 0.4654 0.5953
+
     """
 
     # Group by data vars
@@ -674,6 +748,7 @@ def auto_combine(
              'no_conflicts', 'override'}, optional
         String indicating how to compare variables of the same name for
         potential conflicts:
+
         - 'broadcast_equals': all values must be equal when variables are
           broadcast against each other to ensure common dimensions.
         - 'equals': all values and dimensions must be the same.
@@ -716,7 +791,7 @@ def auto_combine(
     if not from_openmfds:
         basic_msg = dedent(
             """\
-        In xarray version 0.14 `auto_combine` will be deprecated. See
+        In xarray version 0.15 `auto_combine` will be deprecated. See
         http://xarray.pydata.org/en/stable/combining.html#combining-multi"""
         )
         warnings.warn(basic_msg, FutureWarning, stacklevel=2)
@@ -758,7 +833,7 @@ def auto_combine(
         message += dedent(
             """\
         The datasets supplied require both concatenation and merging. From
-        xarray version 0.14 this will operation will require either using the
+        xarray version 0.15 this will operation will require either using the
         new `combine_nested` function (or the `combine='nested'` option to
         open_mfdataset), with a nested list structure such that you can combine
         along the dimensions {}. Alternatively if your datasets have global
@@ -881,7 +956,7 @@ def _auto_concat(
                     "supply the ``concat_dim`` argument "
                     "explicitly"
                 )
-            dim, = concat_dims
+            (dim,) = concat_dims
         return concat(
             datasets,
             dim=dim,
