@@ -1,7 +1,7 @@
 import collections.abc
-from collections import OrderedDict
-from typing import Any, Hashable, Iterable, Mapping, Optional, Tuple, Union
+from typing import Any, Dict, Hashable, Iterable, Mapping, Optional, Tuple, Union
 
+import numpy as np
 import pandas as pd
 
 from . import formatting
@@ -35,13 +35,16 @@ class Indexes(collections.abc.Mapping):
     def __getitem__(self, key):
         return self._indexes[key]
 
+    def __delitem__(self, key):
+        del self._indexes[key]
+
     def __repr__(self):
         return formatting.indexes_repr(self)
 
 
 def default_indexes(
     coords: Mapping[Any, Variable], dims: Iterable
-) -> "OrderedDict[Any, pd.Index]":
+) -> Dict[Hashable, pd.Index]:
     """Default indexes for a Dataset/DataArray.
 
     Parameters
@@ -56,14 +59,14 @@ def default_indexes(
     Mapping from indexing keys (levels/dimension names) to indexes used for
     indexing along that dimension.
     """
-    return OrderedDict((key, coords[key].to_index()) for key in dims if key in coords)
+    return {key: coords[key].to_index() for key in dims if key in coords}
 
 
 def isel_variable_and_index(
     name: Hashable,
     variable: Variable,
     index: pd.Index,
-    indexers: Mapping[Any, Union[slice, Variable]],
+    indexers: Mapping[Hashable, Union[int, slice, np.ndarray, Variable]],
 ) -> Tuple[Variable, Optional[pd.Index]]:
     """Index a Variable and pandas.Index together."""
     if not indexers:
