@@ -294,7 +294,7 @@ class FacetGrid:
                     hue=hue,
                     add_legend=False,
                     _labels=False,
-                    **kwargs
+                    **kwargs,
                 )
                 self._mappables.append(mappable)
 
@@ -376,7 +376,7 @@ class FacetGrid:
             labels=list(self._hue_var.values),
             title=self._hue_label,
             loc="center right",
-            **kwargs
+            **kwargs,
         )
 
         self.figlegend = figlegend
@@ -491,7 +491,7 @@ class FacetGrid:
                     rotation=270,
                     ha="left",
                     va="center",
-                    **kwargs
+                    **kwargs,
                 )
 
             # The column titles on the top row
@@ -590,7 +590,7 @@ def _easy_facetgrid(
     subplot_kws=None,
     ax=None,
     figsize=None,
-    **kwargs
+    **kwargs,
 ):
     """
     Convenience method to call xarray.plot.FacetGrid from 2d plotting methods
