@@ -1,5 +1,4 @@
 import warnings
-from distutils.version import LooseVersion
 from textwrap import dedent
 
 import numpy as np
@@ -28,7 +27,6 @@ from . import (
     arm_xfail,
     assert_array_equal,
     has_dask,
-    has_np113,
     raises_regex,
     requires_cftime,
     requires_dask,
@@ -276,23 +274,39 @@ def assert_dask_array(da, dask):
 
 
 @arm_xfail
-@pytest.mark.parametrize("dask", [False, True])
-def test_datetime_reduce(dask):
-    time = np.array(pd.date_range("15/12/1999", periods=11))
-    time[8:11] = np.nan
-    da = DataArray(np.linspace(0, 365, num=11), dims="time", coords={"time": time})
+@pytest.mark.parametrize("dask", [False, True] if has_dask else [False])
+def test_datetime_mean(dask):
+    # Note: only testing numpy, as dask is broken upstream
+    da = DataArray(
+        np.array(["2010-01-01", "NaT", "2010-01-03", "NaT", "NaT"], dtype="M8"),
+        dims=["time"],
+    )
+    if dask:
+        # Trigger use case where a chunk is full of NaT
+        da = da.chunk({"time": 3})
 
-    if dask and has_dask:
-        chunks = {"time": 5}
-        da = da.chunk(chunks)
+    expect = DataArray(np.array("2010-01-02", dtype="M8"))
+    expect_nat = DataArray(np.array("NaT", dtype="M8"))
 
-    actual = da["time"].mean()
-    assert not pd.isnull(actual)
-    actual = da["time"].mean(skipna=False)
-    assert pd.isnull(actual)
+    actual = da.mean()
+    if dask:
+        assert actual.chunks is not None
+    assert_equal(actual, expect)
 
-    # test for a 0d array
-    assert da["time"][0].mean() == da["time"][:1].mean()
+    actual = da.mean(skipna=False)
+    if dask:
+        assert actual.chunks is not None
+    assert_equal(actual, expect_nat)
+
+    # tests for 1d array full of NaT
+    assert_equal(da[[1]].mean(), expect_nat)
+    assert_equal(da[[1]].mean(skipna=False), expect_nat)
+
+    # tests for a 0d array
+    assert_equal(da[0].mean(), da[0])
+    assert_equal(da[0].mean(skipna=False), da[0])
+    assert_equal(da[1].mean(), expect_nat)
+    assert_equal(da[1].mean(skipna=False), expect_nat)
 
 
 @requires_cftime
@@ -353,11 +367,11 @@ def test_reduce(dim_num, dtype, dask, func, skipna, aggdim):
         warnings.filterwarnings("ignore", "All-NaN slice")
         warnings.filterwarnings("ignore", "invalid value encountered in")
 
-        if has_np113 and da.dtype.kind == "O" and skipna:
+        if da.dtype.kind == "O" and skipna:
             # Numpy < 1.13 does not handle object-type array.
             try:
                 if skipna:
-                    expected = getattr(np, "nan{}".format(func))(da.values, axis=axis)
+                    expected = getattr(np, f"nan{func}")(da.values, axis=axis)
                 else:
                     expected = getattr(np, func)(da.values, axis=axis)
 
@@ -402,7 +416,7 @@ def test_reduce(dim_num, dtype, dask, func, skipna, aggdim):
         actual = getattr(da, func)(skipna=skipna)
         if dask:
             assert isinstance(da.data, dask_array_type)
-        expected = getattr(np, "nan{}".format(func))(da.values)
+        expected = getattr(np, f"nan{func}")(da.values)
         if actual.dtype == object:
             assert actual.values == np.array(expected)
         else:
@@ -442,7 +456,10 @@ def test_argmin_max(dim_num, dtype, contains_nan, dask, func, skipna, aggdim):
             **{aggdim: getattr(da, "arg" + func)(dim=aggdim, skipna=skipna).compute()}
         )
         expected = getattr(da, func)(dim=aggdim, skipna=skipna)
-        assert_allclose(actual.drop(actual.coords), expected.drop(expected.coords))
+        assert_allclose(
+            actual.drop_vars(list(actual.coords)),
+            expected.drop_vars(list(expected.coords)),
+        )
 
 
 def test_argmin_max_error():
@@ -531,12 +548,8 @@ def test_min_count(dim_num, dtype, dask, func, aggdim):
     min_count = 3
 
     actual = getattr(da, func)(dim=aggdim, skipna=True, min_count=min_count)
-
-    if LooseVersion(pd.__version__) >= LooseVersion("0.22.0"):
-        # min_count is only implenented in pandas > 0.22
-        expected = series_reduce(da, func, skipna=True, dim=aggdim, min_count=min_count)
-        assert_allclose(actual, expected)
-
+    expected = series_reduce(da, func, skipna=True, dim=aggdim, min_count=min_count)
+    assert_allclose(actual, expected)
     assert_dask_array(actual, dask)
 
 
