@@ -1,8 +1,6 @@
 import warnings
-from collections import OrderedDict
 from copy import copy, deepcopy
 from datetime import datetime, timedelta
-from distutils.version import LooseVersion
 from textwrap import dedent
 
 import numpy as np
@@ -35,6 +33,7 @@ from . import (
     assert_identical,
     raises_regex,
     requires_dask,
+    requires_sparse,
     source_ndarray,
 )
 
@@ -60,7 +59,7 @@ class VariableSubclassobjects:
         attrs = {"foo": "bar"}
         v.attrs = attrs
         assert v.attrs == attrs
-        assert isinstance(v.attrs, OrderedDict)
+        assert isinstance(v.attrs, dict)
         v.attrs["foo"] = "baz"
         assert v.attrs["foo"] == "baz"
 
@@ -101,7 +100,7 @@ class VariableSubclassobjects:
         ind = Variable(("a", "b"), [[0, 1], [0, 1]])
         v_new = v[ind]
         assert v_new.dims == ("a", "b")
-        expected = np.array(v._data)[([0, 1], [0, 1]),]  # noqa
+        expected = np.array(v._data)[([0, 1], [0, 1]), ...]
         assert_array_equal(v_new, expected)
 
         # boolean indexing
@@ -217,7 +216,7 @@ class VariableSubclassobjects:
                 return hash(self.item)
 
             def __repr__(self):
-                return "%s(item=%r)" % (type(self).__name__, self.item)
+                return "{}(item={!r})".format(type(self).__name__, self.item)
 
         item = HashableItemWrapper((1, 2, 3))
         x = self.cls("x", [item])
@@ -544,6 +543,15 @@ class VariableSubclassobjects:
         with raises_regex(ValueError, "must match shape of object"):
             orig.copy(data=new_data)
 
+    def test_replace(self):
+        var = Variable(("x", "y"), [[1.5, 2.0], [3.1, 4.3]], {"foo": "bar"})
+        result = var._replace()
+        assert_identical(result, var)
+
+        new_data = np.arange(4).reshape(2, 2)
+        result = var._replace(data=new_data)
+        assert_array_equal(result.data, new_data)
+
     def test_real_and_imag(self):
         v = self.cls("x", np.arange(3) - 1j * np.arange(3), {"foo": "bar"})
         expected_re = self.cls("x", np.arange(3), {"foo": "bar"})
@@ -1282,6 +1290,9 @@ class TestVariable(VariableSubclassobjects):
         w2 = Variable(["d", "b", "c", "a"], np.einsum("abcd->dbca", x))
         assert w2.shape == (5, 3, 4, 2)
         assert_identical(w2, w.transpose("d", "b", "c", "a"))
+        assert_identical(w2, w.transpose("d", ..., "a"))
+        assert_identical(w2, w.transpose("d", "b", "c", ...))
+        assert_identical(w2, w.transpose(..., "b", "c", "a"))
         assert_identical(w, w2.transpose("a", "b", "c", "d"))
         w3 = Variable(["b", "c", "d", "a"], np.einsum("abcd->bcda", x))
         assert_identical(w, w3.transpose("a", "b", "c", "d"))
@@ -1331,7 +1342,7 @@ class TestVariable(VariableSubclassobjects):
         actual = v.set_dims(["y", "x"])
         assert_identical(actual, expected.T)
 
-        actual = v.set_dims(OrderedDict([("x", 2), ("y", 2)]))
+        actual = v.set_dims({"x": 2, "y": 2})
         expected = Variable(["x", "y"], [[0, 0], [1, 1]])
         assert_identical(actual, expected)
 
@@ -1379,15 +1390,15 @@ class TestVariable(VariableSubclassobjects):
 
     def test_unstack(self):
         v = Variable("z", [0, 1, 2, 3], {"foo": "bar"})
-        actual = v.unstack(z=OrderedDict([("x", 2), ("y", 2)]))
+        actual = v.unstack(z={"x": 2, "y": 2})
         expected = Variable(("x", "y"), [[0, 1], [2, 3]], v.attrs)
         assert_identical(actual, expected)
 
-        actual = v.unstack(z=OrderedDict([("x", 4), ("y", 1)]))
+        actual = v.unstack(z={"x": 4, "y": 1})
         expected = Variable(("x", "y"), [[0], [1], [2], [3]], v.attrs)
         assert_identical(actual, expected)
 
-        actual = v.unstack(z=OrderedDict([("x", 4)]))
+        actual = v.unstack(z={"x": 4})
         expected = Variable("x", [0, 1, 2, 3], v.attrs)
         assert_identical(actual, expected)
 
@@ -1412,7 +1423,7 @@ class TestVariable(VariableSubclassobjects):
 
     def test_stack_unstack_consistency(self):
         v = Variable(["x", "y"], [[0, 1], [2, 3]])
-        actual = v.stack(z=("x", "y")).unstack(z=OrderedDict([("x", 2), ("y", 2)]))
+        actual = v.stack(z=("x", "y")).unstack(z={"x": 2, "y": 2})
         assert_identical(actual, v)
 
     def test_broadcasting_math(self):
@@ -1476,6 +1487,10 @@ class TestVariable(VariableSubclassobjects):
 
         with raises_regex(ValueError, "cannot supply both"):
             v.mean(dim="x", axis=0)
+        with pytest.warns(DeprecationWarning, match="allow_lazy is deprecated"):
+            v.mean(dim="x", allow_lazy=True)
+        with pytest.warns(DeprecationWarning, match="allow_lazy is deprecated"):
+            v.mean(dim="x", allow_lazy=False)
 
     def test_quantile(self):
         v = Variable(["x", "y"], self.d)
@@ -1608,7 +1623,7 @@ class TestVariable(VariableSubclassobjects):
         # Test dropped attrs
         vm = v.mean()
         assert len(vm.attrs) == 0
-        assert vm.attrs == OrderedDict()
+        assert vm.attrs == {}
 
         # Test kept attrs
         vm = v.mean(keep_attrs=True)
@@ -1621,7 +1636,7 @@ class TestVariable(VariableSubclassobjects):
         b = Variable(["x", "y"], np.random.randn(3, 3), _attrs)
         # Test dropped attrs
         d = a - b  # just one operation
-        assert d.attrs == OrderedDict()
+        assert d.attrs == {}
         # Test kept attrs
         with set_options(keep_attrs=True):
             d = a - b
@@ -1837,13 +1852,6 @@ class TestVariableWithDask(VariableSubclassobjects):
     def test_getitem_1d_fancy(self):
         super().test_getitem_1d_fancy()
 
-    def test_equals_all_dtypes(self):
-        import dask
-
-        if "0.18.2" <= LooseVersion(dask.__version__) < "0.19.1":
-            pytest.xfail("https://github.com/pydata/xarray/issues/2318")
-        super().test_equals_all_dtypes()
-
     def test_getitem_with_mask_nd_indexer(self):
         import dask.array as da
 
@@ -1855,6 +1863,17 @@ class TestVariableWithDask(VariableSubclassobjects):
         )
 
 
+@requires_sparse
+class TestVariableWithSparse:
+    # TODO inherit VariableSubclassobjects to cover more tests
+
+    def test_as_sparse(self):
+        data = np.arange(12).reshape(3, 4)
+        var = Variable(("x", "y"), data)._as_sparse(fill_value=-1)
+        actual = var._to_dense()
+        assert_identical(var, actual)
+
+
 class TestIndexVariable(VariableSubclassobjects):
     cls = staticmethod(IndexVariable)
 
